'''
TensorFlow入門
https://qiita.com/horiem/items/00ec6488b23895cc4fe2

"tf-2.add.py"では入力データを手打ちで入力していた。
しかし、これでは汎用性がないため、"Placeholder"を使用して、動的に入力可能にした。
'''

import tensorflow as tf


# Step 1. データの準備
# 動的に入力するため、事前準備は行わない

# Step 2. 演算の定義
'''
"tf-2.add.py"で演算の定義のために使用していた"tf.constant"ではなく、
"tf.placeholder"に変更した。
placeholderは引数には入力されるデータの型だけを代入しておけば良く、
指定した方のデータを後から動的に入力することができる。

つまり、ここでは"tf.int32"型の整数v1およびv2を定義した、ということ。
'''
v1 = tf.placeholder(tf.int32)
v2 = tf.placeholder(tf.int32)
add = v1 + v2


# Step 3. 演算の実行
'''
"tf-2.add.py"では既に入力データが宣言されていたため、
"print(session.run(add))"だけで演算が実行されていたが、
今回はデータを動的に入力するためにplaceholderを使用しているので、
演算を実行する際にデータを入力する必要がある。
その際には辞書型でデータを入力する。
なお、定義の段階では整数であることしか宣言していなかったが、
整数であれば、入力が複数(＝リスト)であっても演算できる。
ただし、当然ではあるがリストの長さは同じでなければならない。
'''
session = tf.Session()
print(session.run(add, {v1: 4, v2: 12}))  # => 16
print(session.run(add, {v1: 400, v2: 12}))  # => 412
print(session.run(add, {v1: [1, 2], v2: [3, 4]}))  # => [4 6]
print(session.run(add, {v1: [1, 2], v2: [3, 4, 5]}))  # => Error