'''
TensorFlow入門
https://qiita.com/horiem/items/00ec6488b23895cc4fe2

線形関数のフィッティングを実行する。
今回はフィッティング関数として「平均二乗誤差平方根」を用いる。
正解の線形関数：y_answer=a_answer*x+b_answer
学習していく線形関数：y_model=a_model*x+b_model
'''

import numpy as np
import tensorflow as tf


# Step 1. データの準備
'''
np.linspace()=等間隔にデータを準備する関数。
ここでは区間0.0~1.0の間に6個のデータを等間隔に得ている。
'''
x_data = np.linspace(0., 1., 6)  # => => [0.0 0.2 0.4 0.6 0.8 1.0]
a_answer = 1.5
b_answer = .1
y_data = a_answer * x_data + b_answer  # => [0.1 0.4 0.7 1.0 1.3 1.6]

# Step 2. 演算の定義
'''
データを入力するための x 、 答えの y の値を入力するための y_answer は Placeholderで定義。
つまり、計算を実行するときに必要な値を入れる。 
'''
x = tf.placeholder(tf.float32)
y_answer = tf.placeholder(tf.float32)

'''
a_modelやb_modelが学習で最適化したいパラメータであり、これをtf.Variableで定義している。
ここではとりあえず、a_modelの初期値を1.0、b_modelの初期値を0.0とした。
VariableはTensorFlowにおいて、学習モデルのパラメータを定義するために用意されたもの。
学習とはすなわちこのVariableたちの最適な値を見つけることである。
'''
a_model = tf.Variable(1.0)
b_model = tf.Variable(0.0)

'''
y_model = a_model * x + b_model の部分はそのままの意味で、y_modelを定義。
次のlossは上式の誤差を計算する。tf.reduce_mean は、その引数の平均を求める関数。
この際、y_modelやy_answerは配列であることを想定している。

なお、tf.reduce_meanの"reduce"は、次元を減らすという意味。
'''
y_model = a_model * x + b_model
loss = tf.sqrt(tf.reduce_mean((y_model - y_answer) ** 2))

'''
ここは「勾配降下法（GradientDescent）を用いて、
誤差 (loss) を最小にせよ (minimize) 」という部分。
GradientDescentOptimizer(0.0001) の引数 0.0001 は 学習率 と呼ばれるもので、
誤差をどれくらいのスピードで小さくしていくかを調整する値。
小さすぎると学習の収束（loss が最小の状態に近づくこと）が遅くなるが、
大きすぎるとパラメータが最適値に落ちにくくなる（偽の最適値に捕まってしまう）。
'''
train = tf.train.GradientDescentOptimizer(0.0001).minimize(loss)

# Variableを使用する際の初期化に必要
init = tf.global_variables_initializer()


# Step 3. 演算の実行
# まずはsessionのインスタンス化
session = tf.Session()

# 次に、Variableの初期化を実行
session.run(init)

'''
ここからが本番の学習。
上で定義したtrainという計算は20000回実行する。
ここでは誤差(loss)を最小にするために20000回モデルの更新を行っている。
'''
for i in range(20000):
    session.run(train, {x: x_data, y_answer: y_data})

    '''
    結果を表示する部分。
    1000epochにつき1回、結果を出力する。
    '''
    if i % 1000 == 0:
        current_loss, current_y_model = session.run(
            [loss, y_model], {x: x_data, y_answer: y_data})
        print(f"epoch: {i}")
        print(f"Loss: {current_loss}")
        print(f"y_model: {current_y_model}")
        print(f"y_answer: {y_data}" + "\n")
