'''
TensorFlow入門
https://qiita.com/horiem/items/00ec6488b23895cc4fe2

"tf-4.linear_fanc.py"を変形し、2変数線形関数のフィッティングを実行する。
これを応用すると、画像のような配列を入力に使用できる。
それ以外は"tf-4.linear_fanc.py"と同じ。
正解の線形関数：y_answer=W_answer*x+b_answer
学習していく線形関数：y_model=W_model*x+b_model
'''

import numpy as np
import tensorflow as tf


def init_weight_variable(shape):
    """
    重みを正規乱数で初期化する関数
    """
    initial = tf.truncated_normal(shape, stddev=0.1)
    return tf.Variable(initial)


def init_bias_variable(shape):
    """
    バイアスを定数(今回は、0.1)で初期化する関数
    """
    initial = tf.constant(0.1, shape=shape)
    return tf.Variable(initial)


# Step 1. データの準備
# x_data としてランダムな 2 次元ベクトルを 10 個生成
x_data = np.random.random((10, 2))

# w_answerおよびb_answerとして、正解を定義
w_answer = np.array([[1., 2.], [3., 4.]])
b_answer=np.array([-1., 5.])

# それぞれのベクトルに対してw_answerをかけてb_answerを足すことでy_dataを作成
y_data = np.array([w_answer @ _x_data + b_answer for _x_data in x_data])

# Step 2. 演算の定義
# Noneは何個来るのかわからないときに使用する。
x = tf.placeholder(tf.float32, (None, 2))
y_answer = tf.placeholder(tf.float32)

w = init_weight_variable((2, 2))
b=init_bias_variable((2,))

# 行列の計算「W × x」を実行する
y_model = tf.matmul(x, w, transpose_b=True) + b

loss=tf.sqrt(tf.reduce_mean((y_model - y_answer)** 2))

# 最適化関数にAdamを使用
train = tf.train.AdamOptimizer().minimize(loss)

init = tf.global_variables_initializer()

# Step 3. 演算の実行
session = tf.Session()
session.run(init)
for i in range(20000):
    session.run(train, {x: x_data, y_answer: y_data})
    if i % 1000 == 0:
        current_loss, current_w, current_b = session.run(
            [loss, w, b], {x: x_data, y_answer: y_data})
        print(f"epoch: {i}")
        print(f"Loss: {current_loss}")
        print(f"W: {current_w}")
        print(f"b: {current_b}" + "\n")