'''
TensorFlow入門
https://qiita.com/horiem/items/00ec6488b23895cc4fe2

DNNの実装。
1変数の非線形関数をフィッティングする。
'''

import matplotlib.pyplot as plt
import numpy as np
import tensorflow as tf


def init_weight_variable(shape):
    """
    重みを正規乱数で初期化する関数
    """
    initial = tf.truncated_normal(shape, stddev=0.1)
    return tf.Variable(initial)


def init_bias_variable(shape):
    """
    バイアスを定数(今回は、0.1)で初期化する関数
    """
    initial = tf.constant(0.1, shape=shape)
    return tf.Variable(initial)


# Step 1. データの準備
# サイン関数y=sin(x)を入力として定義している
x_data = np.linspace(0., 8 * np.pi, 100)[:, None]
y_data = np.sin(x_data)

# Step 2. 演算の定義
x = tf.placeholder(tf.float32, (None, 1))
y_answer = tf.placeholder(tf.float32)

n_var = 150

w1 = init_weight_variable((1, n_var))
b1 = init_bias_variable((n_var,))
h1 = tf.nn.relu(tf.matmul(x, w1) + b1)

w2 = init_weight_variable((n_var, n_var))
b2 = init_bias_variable((n_var,))
h2 = tf.nn.relu(tf.matmul(h1, w2) + b2)

w3 = init_weight_variable((n_var, n_var))
b3 = init_bias_variable((n_var,))
h3 = tf.nn.relu(tf.matmul(h2, w3) + b3)

w4 = init_weight_variable((n_var, n_var))
b4 = init_bias_variable((n_var,))
h4 = tf.nn.relu(tf.matmul(h3, w4) + b4)

w5 = init_weight_variable((n_var, 1))
b5 = init_bias_variable((1,))
y_model = tf.matmul(h4, w5) + b5

loss = tf.reduce_mean((y_model - y_answer)**2)
train = tf.train.AdamOptimizer().minimize(loss)

init = tf.global_variables_initializer()

# Step 3. 演算の実行
session = tf.Session()
session.run(init)
for i in range(10000):
    session.run(train, {x: x_data, y_answer: y_data})
    if i % 1000 == 0:
        current_loss, current_y_model = session.run(
            [loss, y_model], {x: x_data, y_answer: y_data})
        print(f"Loss: {current_loss}")
current_loss, current_y_model = session.run(
    [loss, y_model], {x: x_data, y_answer: y_data})

plt.plot(x_data, y_data, '.-', label='Answer')
plt.plot(x_data, current_y_model, '.', label='Model')
plt.legend()
plt.show()
